%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program solves the growth model based on ad hoc LQ approximations
%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Jianjun Miao, Department of Economics, Boston University
% Email: miaoj@bu.edu 
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%clear all
%close all
tic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

alpha   = 0.33;
beta    = 0.99;
k = (alpha*beta)^(1/(1-alpha));
c = (1-alpha*beta)*k^alpha;
y = k^alpha;


c_bar = c; r = alpha*k^(alpha-1); %gross interest rate
rho1 = 0.9; rho2 = 0.5;
sigma1 = 0.01; sigma2 = 0.05;

L = zeros(3,2);
L(2,1) = sigma1;
L(3,2) = sigma2;
W = L*L';

A = [r y y; 0 rho1 0; 0 0 rho2];

A_h = [1 0 0 0;0 r y y; 0 0 rho1 0; 0 0 0 rho2]; %Augmented state transition matrix by including a constant state of 1

n = 4; %Number of states (1,k_t,a_{1t},a_{2t})


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Full Information Solution
% LQG parameter 
% Include a constant state of 1 
disp('**************************************************')
disp('Full Information Solution')
Q = zeros(n,n);
Q(1,1) = -log(c);
S = zeros(n,1);
S(1,1) = -0.5/c;
R = 0.5/c^2;
B = [0;-1;0;0]; 

% Riccatti Equation
 P0 = eye(n,n); 
 [P,F, err] = LQG(A_h,B,S,Q,R,beta,P0);
  
  Omega = F'*(R+beta*B'*P*B)*F;
  
  % truncate constant state
  Omega = Omega(2:end,2:end);
  P = P(2:end,2:end);
  B = B(2:end,:);
  disp('Our consumption policy: c = -F*xhat')
  F = F(:,2:end)  
  
  %% Check Luo's (2008) closed-form decision rule using a single state variable
  %disp('Luo (2008) consumption policy: c = -F*xhat')
  %F = -[(r-1)/r, ((r-1)/r)*(rho_1/(r-rho_1)),((r-1)/r)*(rho_2/(r-rho_2))] 
  %Check Luo's (2008) signal weight
  %CL = [1, rho_1/(r-rho_1), rho_2/(r-rho_2)];
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Solutions under RI, three case with different information costs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('**************************************************')
disp('RI Solutions')

disp('-----------------------------------')
lambda = 0.005; 
disp('-----------------------------------')

    Sigma_pr0 =  A*A'+W;
    Theta0 = Omega;  
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0);%Steady state 
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute steady state optimal signal structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[C,V] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Call a Matlab function in our toolbox

c_1 = C(1,1);
C = C./c_1;
V = V/c_1^2;

disp('Our steady-state signal solution: [C V]')
[C V]

Period = 100;
%Call a Matlab function in our toolbox to general IRF

[X_full, u_full, X, u1] = RI_IRF3(A,B,F,L, Period, SigmaSS, C, V);

toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot figures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%set(gcf, 'PaperPositionMode', 'auto')
%set(gcf,'Renderer','painters')
%print -deps Fig6New.eps
%print -dpdf consumption.pdf









  
  
  