%% Durable and Nondurable Consumption Model
%% State vector x_t = (y_{1t},y_{2t},y_{zt},k_{t},b_{t}) and control vector
%% u_{t} = (c_{t},I_{t})

clear all;
close all;
tic
%Model specification
[gx, gu, Hxx, Hxu, Huu] = Durable_model;

%Numerical Evaluation
%Steady State and Parameter Values
[THETA, R, BETTA, GAMA,delta, phik,phib, ybar, bbar, rho1,rho2, sigma1, sigma2, sigmaz, sigmak,...
    c,k, b, yz, Inv, y1, y2, lambdaSS1,lambdaSS2, lambdaSS3, lambdaSS4,lambdaSS5] = Durable_model_ss;

%Obtain numerical derivatives of f
num_derivative

%% Solution under Full information 
%% Using MWY(2022) Notation

Q = -nHxx/2;
S = -nHxu/2;
R = -nHuu/2;
A = ngx;
B = ngu;

nx =size(Q,1);

P0 = eye(nx,nx);
[P,F,err,iter] = LQG(A,B,S,Q,R,BETTA,P0); % Call a function to compute Riccati equation
disp('Linear decision rule coefficents: ')
% u = -F*\hat{x}; 
F

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%RI solution
%Shocks volatility matrix
L = zeros(5,4);
L(1,1) = sigma1;
L(2,2) = sigma2;
L(3,3) = sigmaz;
L(4,4) = sigmak;

W = L*L';

lambda= 0.005; %Information cost parameter
Omega = F'*(R+ BETTA*B'*P*B)*F;

    Sigma_pr0 = A*A'+W; %Initial guess of optimal prior covariance matrix 
    
    Theta0 = Omega; % Initial guess of dynamic benefit matrix
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,BETTA,A,W,Omega, Theta0,Sigma_pr0); 

[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Compute signal structure

display('Steady state signal structure: [C V], Sigma')
[C,V]
SigmaSS


%impulse responses to one standard deviation innovation shock

Period = 50;
[X_full, u_full, X, u] = RI_IRF3(A,B,F,L, Period, SigmaSS, C, V);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Transform to log linear responses to 1% shocks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
toc
%IRFs to y_{1t} shock
figure;
time =1:Period;


subplot(2,2,1)
plot(time,u_full(1,:,1)/c/sigma1, 'LineWidth',1.5);
    hold on
plot(time,u(1,:,1)/c/sigma1,'--','LineWidth',1.5);
title('c_t to y_{1t} shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8

subplot(2,2,2)
plot(time,u_full(2,:,1)/Inv/sigma1,'LineWidth',1.5);
    hold on
plot(time,100*u(2,:,1)/Inv,'--','LineWidth',1.5);
title(['I_t to y_{1t} shock'])
legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,2,3)
plot(time(1:end-1),100*X_full(4,2:end,1)/k,'LineWidth',1.5);
    hold on
plot(time(1:end-1),100*X(4,2:end,1)/k,'--','LineWidth',1.5);
title('k_{t+1} to y_{1t} shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;



subplot(2,2,4)
plot(time(1:end-1),100*X_full(5,2:end,1)/bbar,'LineWidth',1.5);
    hold on
plot(time(1:end-1),100*X(5,2:end,1)/bbar,'--','LineWidth',1.5);
xlabel('time')
%legend('Full Info','RI')
title('b_{t+1} to y_{1t} shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

print -depsc FigDurable1.eps

%IRFs to y_{2t} shock
figure
subplot(2,2,1)
plot(time,u_full(1,:,2)/c/sigma2, 'LineWidth',1.5);
    hold on
plot(time,u(1,:,2)/c/sigma2,'--','LineWidth',1.5);
title('c_t to y_{2t} shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8

subplot(2,2,2)
plot(time,u_full(2,:,2)/Inv/sigma2,'LineWidth',1.5);
    hold on
plot(time,u(2,:,2)/Inv/sigma2,'--','LineWidth',1.5);
title(['I_t to y_{2t} shock'])
legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,2,3)
plot(time(1:end-1),X_full(4,2:end,2)/k/sigma2,'LineWidth',1.5);
    hold on
plot(time(1:end-1),X(4,2:end,2)/k/sigma2,'--','LineWidth',1.5);
title('k_{t+1} to y_{2t} shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;



subplot(2,2,4)
plot(time(1:end-1),X_full(5,2:end,2)/bbar/sigma2,'LineWidth',1.5);
    hold on
plot(time(1:end-1),X(5,2:end,2)/bbar/sigma2,'--','LineWidth',1.5);
xlabel('time')
%legend('Full Info','RI')
title('b_{t+1} to y_{2t} shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

print -depsc FigDurable2.eps

%IRFs to purely temporary z_t shock
figure
subplot(2,2,1)
plot(time,u_full(1,:,3)/c/sigmaz, 'LineWidth',1.5);
    hold on
plot(time,u(1,:,3)/c/sigmaz,'--','LineWidth',1.5);
title('c_t to \epsilon_{zt} shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8

subplot(2,2,2)
plot(time,u_full(2,:,3)/Inv/sigmaz,'LineWidth',1.5);
    hold on
plot(time,u(2,:,3)/Inv/sigmaz,'--','LineWidth',1.5);
title(['I_t to \epsilon_{zt} shock'])
legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

subplot(2,2,3)
plot(time(1:end-1),X_full(4,2:end,3)/k/sigmaz,'LineWidth',1.5);
    hold on
plot(time(1:end-1),X(4,2:end,3)/k/sigmaz,'--','LineWidth',1.5);
title('k_{t+1} to \epsilon_{zt} shock')
%legend('Full Info','RI')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;



subplot(2,2,4)
plot(time(1:end-1),X_full(5,2:end,3)/bbar/sigmaz,'LineWidth',1.5);
    hold on
plot(time(1:end-1),X(5,2:end,3)/bbar/sigmaz,'--','LineWidth',1.5);
xlabel('time')
%legend('Full Info','RI')
title('b_{t+1} to \epsilon_{zt} shock')
ax = gca;
ax.FontSize = 7;
ax.TitleFontSizeMultiplier = 0.8;

print -depsc FigDurable3.eps